/*
 * kma36.c
 *
 *  Created on: Jul 30, 2014
 *      Author: JMBrinkhus
 */

#include "kma36.h"
#include "xiic_l.h"
#include "sleep.h"
#include "math.h"
#include "xparameters.h"

// KMA36 Global Variables
u32 			kma36_axi_address;
kma36_address 	kma36_i2c_address;
u16				kma36_ma;
u32				kma36_ilc;
u8				kma36_kconf;
u8				kma36_kresh;
u8				kma36_kresl;

/*********************************************************************
 *
 * Function:    kma36_init
 *
 * Description:	Initialize the axi address
 *
 * Parameters:	u32 axi_address -
 * 					Address of Xilinx AXI IIC Peripheral
 *
 * Returns:		void
 *
 *********************************************************************/
void kma36_init(u32 axi_address){

	kma36_axi_address = axi_address;
	kma36_i2c_address = kma36_i2c_address_GND;
	kma36_ma = 0;
	kma36_ilc = 0;
	kma36_kconf = 0x02;
	kma36_kresh = 0x03;
	kma36_kresl = 0xFF;

	return;

}

/*********************************************************************
 *
 * Function:    kma36_set_i2c_address
 *
 * Description:	Sets the I2C address of the KMA36
 *
 * Parameters:	kma36_address addr -
 * 					Enumerated type passes value of either
 * 					address 0x59, 0x5A, 0x5B, 0x5C, or 0x5D
 *
 * Returns:		void
 *
 *********************************************************************/
void kma36_set_i2c_address(kma36_address addr){

	kma36_i2c_address = addr;

	return;

}

/*********************************************************************
 *
 * Function:    kma36_read_angle
 *
 * Description:	Read KMA36 current angle
 *
 * Parameters:	float* angle -
 * 					Pointer to angle variable.  This function
 * 					writes the current angle into this location
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_read_angle(float* angle){

	kma36_status kma36_stat;
	unsigned int uint_kres;

	kma36_stat = kma36_read_regs();
	uint_kres = (kma36_kresh<<8) + kma36_kresl;
	if(kma36_stat==kma36_status_ok){
		*angle = kma36_ma*360./(uint_kres+1);
		return kma36_status_ok;
	}else{
		return kma36_status_i2c_transfer_error;
	}

}

/*********************************************************************
 *
 * Function:    kma36_sleep_enter
 *
 * Description:	Request KMA36 to enter sleep mode
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_sleep_enter(void){

	kma36_status stat;

	kma36_kconf |= KMA36_SLP;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_sleep_exit
 *
 * Description:	Request KMA36 to exit sleep mode
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_sleep_exit(void){

	kma36_status stat;

	kma36_kconf &= !KMA36_SLP;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_enable_low_power_mode
 *
 * Description:	Request KMA36 to enable low power mode.  In this mode,
 * 				only 180 measurements are possible
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_enable_low_power_mode(void){

	kma36_status stat;

	kma36_kconf |= KMA36_PWR;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_disable_low_power_mode
 *
 * Description:	Request KMA36 to disable low power mode.
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_disable_low_power_mode(void){

	kma36_status stat;

	kma36_kconf &= !KMA36_PWR;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_enable_counter
 *
 * Description:	Request KMA36 to enable full turn counting
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_enable_counter(void){

	kma36_status stat;

	kma36_kconf |= KMA36_CNT;
	stat = kma36_write_regs();

	return stat;
}

/*********************************************************************
 *
 * Function:    kma36_disable_counter
 *
 * Description:	Request KMA36 to disable full turn counting
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_disable_counter(void){

	kma36_status stat;

	kma36_kconf &= !KMA36_CNT;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_enable_fast_rate
 *
 * Description:	Request KMA36 to enable fast measurement update rate.
 * 				In fast mode, measurement accuracy is reduced.
 * 				Update rate = 1/(1.4ms x oversampling / const)
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_enable_fast_rate(void){

	kma36_status stat;

	kma36_kconf |= KMA36_SPD;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_disable_fast_rate
 *
 * Description:	Request KMA36 to disable fast measurement update rate.
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_disable_fast_rate(void){

	kma36_status stat;

	kma36_kconf &= !KMA36_SPD;
	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_set_accuracy
 *
 * Description:	Set KMA36 accuracy.  Oversampling impacts the
 * 				measurement	update rate.  Update rate =
 * 				1/(1.4ms x oversampling / const)
 *
 * Parameters:	kma36_oversampling os -
 * 					Enumerated type passes one of the following values:
 * 					kma36_oversampling_2, kma36_oversampling_4,
 * 					kma36_oversampling_8, or kma36_oversampling_32
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_set_accuracy(kma36_oversampling os){

	kma36_status stat;

	kma36_kconf &= !KMA36_OVSC;
	if(os==kma36_oversampling_2){
		kma36_kconf |= KMA36_OS2;
	}else if(os==kma36_oversampling_4){
		kma36_kconf |= KMA36_OS4;
	}else if(os==kma36_oversampling_8){
		kma36_kconf |= KMA36_OS8;
	}else{
		kma36_kconf |= KMA36_OS32;
	}

	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_set_resolution
 *
 * Description:	Set KMA36 resolution
 *
 * Parameters:	u16 res -
 * 					Unsigned 16 bit number between 1 and 32767
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status kma36_set_resolution(u16 res){

	kma36_status stat;

	kma36_kresh = (u8)(res>>8);
	kma36_kresl = (u8)(res&0xFF);

	stat = kma36_write_regs();

	return stat;

}

/*********************************************************************
 *
 * Function:    kma36_read_regs
 *
 * Description:	Reads the KMA36 registers and updates the magnetic angle,
 * 				incremental linear counter, and configuration global
 * 				variables and returns the status of the read operation
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status  kma36_read_regs(void){

	char rx_buf[7];
	int byte_count=0;

	byte_count = XIic_Recv(kma36_axi_address, kma36_i2c_address, (u8*)rx_buf, 7, XIIC_STOP);
	if(byte_count!=7){
		printf("Received %d/7 bytes. Transfer error.\n",byte_count);
		return kma36_status_i2c_transfer_error;
	}

	kma36_ma    = (rx_buf[1]<<8) + rx_buf[0];
	kma36_ilc   = (rx_buf[5]<<24) + (rx_buf[4]<<16) + (rx_buf[3]<<8) + rx_buf[2];
	kma36_kconf = rx_buf[6];

	usleep(50*1000);

	return kma36_status_ok;

}

/*********************************************************************
 *
 * Function:    kma36_compute_crc
 *
 * Description:	Reads the KMA36 registers and updates the magnetic angle,
 * 				incremental linear counter, and configuration global
 * 				variables and returns the status of the read operation
 *
 * Parameters:	u8* write_regs -
 * 					Buffer address of the buffer containing the three
 * 					bytes of data to be written to the device.
 *
 * Returns:		u8 result -
 * 					One byte crc code computed from the three bytes of
 * 					data to be written to the KMA36
 *
 *********************************************************************/
u8 kma36_compute_crc(u8* write_regs){
	u8 result;
	result = write_regs[0]+write_regs[1]+write_regs[2];
	result = 0xFF-result+0x01;
	return result;
}

/*********************************************************************
 *
 * Function:    kma36_write_regs
 *
 * Description:	Writes the KMA36 registers with the values contained
 * 				in the global configuration and resolution variables.
 * 				Returns the status of the write operation
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type kma36_status with possible values
 * 				kma36_status_ok or kma36_status_i2c_transfer_error
 *
 *********************************************************************/
kma36_status  kma36_write_regs(void){

	u8 tx_buf[4];
	int byte_count=0;

	tx_buf[0] = kma36_kconf;
	tx_buf[1] = kma36_kresh;
	tx_buf[2] = kma36_kresl;
	tx_buf[3] = kma36_compute_crc(tx_buf);

	byte_count = XIic_Send(kma36_axi_address, kma36_i2c_address, (u8*)tx_buf, 4, XIIC_STOP);
	if(byte_count!=4){
		printf("Transmitted %d/4 bytes. Transfer error.\n",byte_count);
		return kma36_status_i2c_transfer_error;
	}

	usleep(50*1000);

	return kma36_status_ok;

}
